unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, Spin, ExtCtrls,Math;



type

(*

56 bytes per record.
Zero header

*)
  TDataMeteoRec=packed record
      Date            :TdateTime;  // 8 bytes  Delphi TIME
      AbsPressure     :Single;     // 4 bytes  HPa
      RelativePressure:Single;     // 4 bytes  Hpa
      WindSpeed       :Single;     // 4 bytes  m/s -> if 51m/s = Invalid, max is 50 m/s  so 180 km/h
      WindDirection   :LongWord;   // 4 bytes
      WindGust        :Single;     // 4 bytes  m/s -> if 51m/s = Invalid, max is 50 m/s  so 180 km/h
      AbsoluteRain    :Single;     // 4 bytes  mm
      RelativeRain    :Single;     // 4 bytes  mm

      TemperatureInt  :Single;     // 4 bytes C
      TemperatureExt  :Single;     // 4 bytes C
      HumidityIn      :Single;     // 4 bytes %RH
      HumidityOut     :Single;     // 4 bytes %RH
      NotUsed         :Single;     // 4 bytes
  end;

  TRowSel=record
           Valid:Boolean;
           Index:Integer;
          end;  

  TFormMain = class(TForm)
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    Panel1: TPanel;
    ButtonOpen: TButton;
    StaticTextFile: TStaticText;
    Panel3: TPanel;
    Panel2: TPanel;
    StaticTextNbRec: TStaticText;
    Button3: TButton;
    Button4: TButton;
    SpinEditIndexArr: TSpinEdit;
    SpinEditIndexBegin: TSpinEdit;
    StringGrid: TStringGrid;
    ButtonDelete: TButton;
    LabelS: TLabel;
    procedure ButtonOpenClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BtnSaveDATClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StringGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure ButtonDeleteClick(Sender: TObject);
  private
    { Private declarations }
    procedure UpdateDisplay;
  public
    { Public declarations }
    DataMeteoRec: Array of TDataMeteoRec;
    NbreRec:LongWord;
    RowSel:TRowSel;
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}


Function ConvertToDewPoint(Temp,RH:Double):Double;
Var    D,ES,E:Double;

Const  K=237.7;
       Z=6.11 ;
       C=7.5  ;
Begin
 // Temp : Actual Temperature C
 // RH   : Relative Humidity  %RH
 // Out  : DewPoint Temp      C

 D := (C*Temp)/(K+Temp);
 ES:=Z*Power(10,D);
 E :=(RH*ES)/100;
 D:=ln(E/Z);

 result:=D*K/(C*ln(10)-D);

end;


Function DegToFahrenheit(DegC:Double):Double;
Begin
 Result:=(9*DegC/5)+32;
end;

Function FahrenheitToDeg(DegF:Double):Double;
Begin
 Result:=(DegF-32)*5/9;
end;


Function ConvertToWindChill(Temp,WindSpeed:Double):Double;
Var    F,Mph,D:Double;

Begin
 // wind chill temperature
 // Temp       : Actual Temperature   C
 // WindSpeed  : Wind Speed           km/h
 // Out        : ind Chill Temp       C
 // windchill = 35.74 + 0.6215*t - 35.75*v^0.16 + 0.4275*t*v^0.16

 //WindSpeed:=50;
 //Temp:=5;

 If (WindSpeed<4.8) then
 begin
  Result:=Temp;
 end
 else
 begin
  F  :=DegToFahrenheit(Temp);
  Mph:=WindSpeed/1.609;
  D  :=35.74 + 0.6215*F - 35.75*Power(Mph,0.16) + 0.4275*F*Power(Mph,0.16);
  Result:=FahrenheitToDeg(D);
 end;

end;



procedure TFormMain.UpdateDisplay;
Var

    k,j :LongWord;
    D:Double;

Begin

  StaticTextNbRec.caption:=Format('Amount of records : %d',[NbreRec]);

  SpinEditIndexArr.MaxValue:=NbreRec;
  SpinEditIndexArr.MinValue:=1;
  SpinEditIndexArr.Value   :=NbreRec;


  SpinEditIndexBegin.MaxValue:=NbreRec;
  SpinEditIndexBegin.MinValue:=1;
  SpinEditIndexBegin.Value   :=1;




  StringGrid.Cells[1,0] :='Date';
  StringGrid.Cells[2,0] :='Pression abs. (hPa)';
  StringGrid.Cells[3,0] :='Pressure Rel. (hPa)';
  StringGrid.Cells[4,0] :='Temp Ext. (C)';
  StringGrid.Cells[5,0] :='Humidit Ext. (%)';
  StringGrid.Cells[6,0] :='Temp. Rose (C)';

  StringGrid.Cells[7,0] :='Temp Int. (C)';
  StringGrid.Cells[8,0] :='Humidit Int. (%)';

  StringGrid.Cells[9,0] :='Vitesse du vent (km/h)';
  StringGrid.Cells[10,0]:='Direction du vent ()';
  StringGrid.Cells[11,0]:='Rafale vent (km/h)';

  StringGrid.Cells[12,0]:='Pluie cumule (mm)';
  StringGrid.Cells[13,0]:='Pluie relative (mm)';
  StringGrid.Cells[14,0]:='Temp. Ressentie (C)';


  StringGrid.RowCount:=NbreRec;
  For k:=1 to StringGrid.RowCount-1 do
  begin
   For j:=1 to 14 do
    StringGrid.Cells[k,j]:='';
  end;


  StringGrid.RowCount:=NbreRec+1;
  For k:=1 to NbreRec do
  begin
   StringGrid.Cells[1,k]:=DateTimeToStr(DataMeteoRec[k-1].Date);
   StringGrid.Cells[0,k]:=Format('%d',[k]);
   StringGrid.Cells[2,k]:=Format('%2.1f',[DataMeteoRec[k-1].AbsPressure]);
   StringGrid.Cells[3,k]:=Format('%2.1f',[DataMeteoRec[k-1].RelativePressure]);
   StringGrid.Cells[4,k]:=Format('%2.1f',[DataMeteoRec[k-1].TemperatureExt]);
   StringGrid.Cells[5,k]:=Format('%2.0f',[DataMeteoRec[k-1].HumidityOut]);
   StringGrid.Cells[6,k]:=Format('%2.1f',[ConvertToDewPoint(DataMeteoRec[k-1].TemperatureExt,DataMeteoRec[k-1].HumidityOut)]);


   StringGrid.Cells[7,k]:=Format('%2.1f',[DataMeteoRec[k-1].TemperatureInt]);
   StringGrid.Cells[8,k]:=Format('%2.0f',[DataMeteoRec[k-1].HumidityIn]);

   D:=DataMeteoRec[k-1].WindSpeed*3.6; //50 last value, 51 Invalid va
   If D<=180 then
   begin
    StringGrid.Cells[9,k] :=Format('%2.1f',[D]);
    StringGrid.Cells[10,k]:=Format('%2.1f',[DataMeteoRec[k-1].WindDirection*22.5]);
    StringGrid.Cells[14,k]:=Format('%2.1f',[ConvertToWindChill(DataMeteoRec[k-1].TemperatureExt,D)]);
   end
   else
   begin
    //StringGrid.Cells[9,k] :=Format('%2.1f',[DataMeteoRec[k-1].WindSpeed]);
    StringGrid.Cells[9,k] :='--';
    StringGrid.Cells[10,k]:='--';
    StringGrid.Cells[14,k]:='--';
   end;


   D:=DataMeteoRec[k-1].WindGust*3.6;
   If D<=180 then
    StringGrid.Cells[11,k]:=Format('%2.1f',[D])
   else
    StringGrid.Cells[11,k]:='--';


   StringGrid.Cells[12,k]:=Format('%2.1f',[DataMeteoRec[k-1].AbsoluteRain]);
   StringGrid.Cells[13,k]:=Format('%2.1f',[DataMeteoRec[k-1].RelativeRain]);

  end;
end;



procedure TFormMain.ButtonOpenClick(Sender: TObject);
Var FN:String;
    F :File;
    S,k :LongWord;

begin
 OpenDialog.InitialDir:='C:\depot\';
 If OpenDialog.Execute then
 begin
  FN:=OpenDialog.Filename;
  AssignFile(F,FN);
  StaticTextFile.caption:='File : '+FN;
  Try

  Reset(F,1);
  S:=FileSize(F);
  NbreRec:=S div Sizeof(TDataMeteoRec);
  Setlength(DataMeteoRec,NbreRec);

  For k:=1 to NbreRec do
   BlockRead(F,DataMeteoRec[k-1],Sizeof(TDataMeteoRec));


  StringGrid.Enabled:=True;
  UpdateDisplay;


  Finally
  CloseFile(F);
  end;
 end;
end;


Procedure ADDFileextensionIfNotThere(Const Ext:String;Var S:String);  // Ext=.cpa par ex
var ExtD:String;
    k,i:Integer;
Begin
 // Aucune extension
 If ExtractFileExt(S)='' then
 begin
  S:=S+Ext;         // ajouter
  Exit;
 end;

 // fichiers en extension double ou multiple comme toto.ads.gtd.aee
 k:=0;
 For i:=length(S) downto 1 do
 begin
  inc(k);
  If S[i]='.' then
  begin
    ExtD  :=Copy(S,i,k);
    If AnsiUppercase(Ext)<>AnsiUppercase(ExtD) then
    begin
     S:=S+Ext;         // ajouter
    end;
    break;
  end
 end;
end;


procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Setlength(DataMeteoRec,0);
end;

procedure TFormMain.BtnSaveDATClick(Sender: TObject);
Var Min,Max,k:LongWord;
    FN:String;
    F :File;

begin
  SaveDialog.Filter:='DAT files|*.dat';
  SaveDialog.InitialDir:='C:\Program Files\HeavyWeather';
  Min:=SpinEditIndexBegin.Value;
  Max:=SpinEditIndexArr.Value;

  If SaveDialog.Execute then
  begin
    FN:=SaveDialog.Filename;
    ADDFileextensionIfNotThere('.dat',FN);

    AssignFile(F,FN);
    Rewrite(F,1);
    Try
    For k:=Min to Max do
    begin
     BlockWrite(F,DataMeteoRec[k-1],Sizeof(TDataMeteoRec));
    end;
    Finally
    CloseFile(F);
    end;
  end;
end;



procedure TFormMain.Button4Click(Sender: TObject);
Var Min,Max,k:LongWord;
    FN,S     :String;
    F        :TextFile;
    D        :Double;


Const
    Sep=';';

begin
  SaveDialog.Filter:='CSV files|*.csv';
  SaveDialog.InitialDir:='C:\Program Files\HeavyWeather';
  Min:=SpinEditIndexBegin.Value;
  Max:=SpinEditIndexArr.Value;

  If SaveDialog.Execute then
  begin
    FN:=SaveDialog.Filename;
    ADDFileextensionIfNotThere('.csv',FN);
    AssignFile(F,FN);
    Rewrite(F);
    Try
    Writeln(F,'#;Date;Abs Pressure;Relative Pressure;Ext. temperature (C);Ext. humidity (%RH);'+
                'Int. temperature (C);int. humidity (%RH);Point de rose (C)'+
                'Vitesse du vent (km/h);Direction du vent ();WindChill Temp (C);'+
                'Rafale vent (km/h);'+
                'Pluie cumule (mm);Pluie relative (mm)');

    For k:=Min to Max do
    begin

      S:=Format('%d',[k-Min+1])+ Sep +
         DateTimeToStr(DataMeteoRec[k-1].Date) + Sep +
         Format('%2.1f',[DataMeteoRec[k-1].AbsPressure]) + Sep +
         Format('%2.1f',[DataMeteoRec[k-1].RelativePressure]) + Sep +
         Format('%2.1f',[DataMeteoRec[k-1].TemperatureExt]) + Sep +
         Format('%2.0f',[DataMeteoRec[k-1].HumidityOut]) + Sep +
         Format('%2.1f',[DataMeteoRec[k-1].TemperatureInt]) + Sep +
         Format('%2.0f',[DataMeteoRec[k-1].HumidityIn]) + Sep +
         Format('%2.1f',[ConvertToDewPoint(DataMeteoRec[k-1].TemperatureExt,DataMeteoRec[k-1].HumidityOut)])+ Sep ;

      D:=DataMeteoRec[k-1].WindSpeed*3.6;
      If D<=180 then
      begin
        S:=S+Format('%2.1f',[D])+ Sep;
        S:=S+Format('%2.1f',[DataMeteoRec[k-1].WindDirection*22.5]) + Sep ;
        S:=S+Format('%2.1f',[ConvertToWindChill(DataMeteoRec[k-1].TemperatureExt,D)]) + Sep;
      end
      else
      begin
        S:=S+'--;';
        S:=S+'--;';
        S:=S+'--;';
      end;


      D:=DataMeteoRec[k-1].WindGust*3.6;
      If D<=180 then
       S:=S+Format('%2.1f',[D])+Sep
      else
       S:=S+'--;';


      S:=S+Format('%2.1f',[DataMeteoRec[k-1].AbsoluteRain])+ Sep +
           Format('%2.1f',[DataMeteoRec[k-1].RelativeRain]);


      Writeln(F,S);
    end;
    Finally
    CloseFile(F);
    end;
  end;

end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
 NbreRec:=0;
 RowSel.Valid:=False;
end;

procedure TFormMain.StringGridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
 LabelS.caption:=Format('%d %d',[ACol, ARow]);
 RowSel.Valid:=True;
 RowSel.Index:=Arow;
end;

procedure TFormMain.ButtonDeleteClick(Sender: TObject);
Var k:Integer;
begin
 For k:=RowSel.Index to NbreRec-1 do
 begin
   DataMeteoRec[k-1]:=DataMeteoRec[k];
 end;

 Dec(NbreRec);
 Setlength(DataMeteoRec,NbreRec);
 UpdateDisplay;

end;

end.
